/* SPDX-License-Identifier: GPL-2.0-only OR BSD-2-Clause */

#ifndef _DT_BINDINGS_CLK_QCOM_DSI_PHY_28NM_H
#define _DT_BINDINGS_CLK_QCOM_DSI_PHY_28NM_H

#define DSI_BYTE_PLL_CLK		0
#define DSI_PIXEL_PLL_CLK		1

#endif
